@@ -1,7 +1,7 @@
 /* bsect.c  -  Boot sector handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -10,6 +10,7 @@ source directory.
 */
 
 
+#define _GNU_SOURCE
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/statfs.h>
@@ -17,6 +18,7 @@ source directory.
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <dirent.h>
 
 #ifdef	_SYS_STATFS_H
 #define	_I386_STATFS_H	/* two versions of statfs is not good ... */
@@ -76,7 +78,7 @@ static off_t here2;		/* sector address of second stage loader */
 static int adapter = -1;	/* video adapter in use */
 
 typedef struct Pass {
-    long crc[MAX_PW_CRC];
+    int crc[MAX_PW_CRC];
     char *unique;
     char *label;
     struct Pass *next;
@@ -131,7 +133,7 @@ static char *disallow[] = {
     if (verbose >= 5) printf("check_options: \"%s\"\n", options);
     while (*dis) {
 	here = strstr(options, *dis);
-	if (here) {
+	if (here  &&  (here==options || here[-1] == ' ') ) {
 	    if (here[3] == '=') error=2;
 	    if (here[3] != '=' && (here[4]==' ' || here[4]==0) ) error=2;
 #if !MEM_OKAY
@@ -270,7 +272,7 @@ void pw_file_update(int passw)
     
 	for (walk=pwsave; walk; walk=walk->next) {
 	    fprintf(pw_file, "label=<\"%s\">", walk->label);
-	    for (i=0; i<MAX_PW_CRC; i++) fprintf(pw_file, " 0x%08lX", walk->crc[i]);
+	    for (i=0; i<MAX_PW_CRC; i++) fprintf(pw_file, " 0x%08X", walk->crc[i]);
 	    fprintf(pw_file, "\n");
 	}
     }
@@ -311,12 +313,12 @@ void pw_fill_cache(void)
     if (verbose >=5) printf("end pw_fill_cache\n");
 }
 
-static void hash_password(char *password, long crcval[])
+static void hash_password(char *password, int crcval[])
 {
 #ifdef CRC_PASSWORDS
-   static long poly[] = {CRC_POLY1, CRC_POLY2, CRC_POLY3, CRC_POLY4, CRC_POLY5};
+   static int poly[] = {CRC_POLY1, CRC_POLY2, CRC_POLY3, CRC_POLY4, CRC_POLY5};
 #endif
-	long crc;
+	int crc;
 	int j;
 	int i = strlen(password);
 	
@@ -336,7 +338,7 @@ static void hash_password(char *password, long crcval[])
 #endif
 	    if(verbose >= 2) {
 		if (j==0) printf("Password " PWTYPE " =");
-		printf(" %08lX", crc);
+		printf(" %08X", crc);
 	    }
 	}
 	if (verbose >= 2) printf("\n");
@@ -380,7 +382,7 @@ char *pw_input(void)
 }
 #endif
 
-static void pw_get(char *pass, long crcval[], int option)
+static void pw_get(char *pass, int crcval[], int option)
 {
     PASSWORD *walk;
     char *pass2;
@@ -395,7 +397,7 @@ static void pw_get(char *pass, long crcval[], int option)
     for (walk=pwsave; walk; walk=walk->next) {
 	if (pass == walk->unique ||
 		(!walk->unique && !strcmp(walk->label,label) && (walk->unique=pass)) ) {
-	    memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(long));
+	    memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(int));
 	    return;
 	}
     }
@@ -423,11 +425,11 @@ static void pw_get(char *pass, long crcval[], int option)
     pw_wipe(pass2);  
     hash_password(pass, walk->crc);
     pw_wipe(pass);
-    memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(long));
+    memcpy(crcval, walk->crc, MAX_PW_CRC*sizeof(int));
 }
 
 
-static void retrieve_crc(long crcval[])
+static void retrieve_crc(int crcval[])
 {
     int i;
     char *pass;
@@ -441,7 +443,7 @@ static void retrieve_crc(long crcval[])
 
     if (verbose >= 2) {
 	printf("Password found is");
-	for (i=0; i<MAX_PW_CRC; i++) printf(" %08lX", crcval[i]);
+	for (i=0; i<MAX_PW_CRC; i++) printf(" %08X", crcval[i]);
 	printf("\n");
     }
 }
@@ -525,7 +527,7 @@ static void menu_do_scheme(char *scheme, MENUTABLE *menu)
     /* order of color attributes is:
 	  text, hilighted text, border, title
     */
-#define color(c) ((int)strchr(khar,(int)(c))-(int)khar)
+#define color(c) ((int)(strchr(khar,(int)(c))-khar))
     bg = 0;
     at = &(menu->at_text);
     for (i=0; i<4 && *scheme; i++) {
@@ -568,7 +570,7 @@ static void menu_do_scheme(char *scheme, MENUTABLE *menu)
 
 
 void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
-  int timeout, long raid_offset)
+  int timeout, int raid_offset)
 {
     static char coms[] = "0123";
     static char parity[] = "NnOoEe";
@@ -585,7 +587,7 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
     BITMAPFILEHEADER fhv;
     BITMAPHEADER bmhv;
     BITMAPLILOHEADER lhv;
-    unsigned long timestamp;
+    unsigned int timestamp;
 #ifdef LCF_BUILTIN
     BUILTIN_FILE *loader;
 #else
@@ -704,7 +706,7 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 		!cfg_get_flag(cf_options, "static-bios-codes") ) /* hard disk & not raid master */
 	    bsect.par_1.map_serial_no = serial_no[bios_map - 0x80];
 	if (verbose>=2)
-	    printf("bios_boot = 0x%02X  bios_map = 0x%02X  map==boot = %d  map S/N: %08lX\n",
+	    printf("bios_boot = 0x%02X  bios_map = 0x%02X  map==boot = %d  map S/N: %08X\n",
 		bios_boot, bios_map,
 		!!(bsect.par_1.prompt&FLAG_MAP_ON_BOOT),
 		bsect.par_1.map_serial_no);
@@ -782,8 +784,15 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
     }
 
     if (cfg_get_flag(cf_options,"el-torito-bootable-cd"))
-    param2.flag2 |= FLAG2_EL_TORITO;
-    
+	param2.flag2 |= FLAG2_EL_TORITO;
+    if (cfg_get_flag(cf_options,"unattended")) {
+	param2.flag2 |= FLAG2_UNATTENDED;
+	if (timeout < 0) {
+	    fprintf(errstd,"Warning: UNATTENDED used; setting TIMEOUT to 200 (20 seconds)\n");
+	    timeout = 200;
+	}
+    }
+        
     serial = cfg_get_strg(cf_options,"serial");
     if (serial) {
     	if (!(stage_flags & STAGE_FLAG_SERIAL))
@@ -828,7 +837,7 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 						(int)param2.ser_param);
 	}
 	if (delay < 20 && !cfg_get_flag(cf_options,"prompt")) {
-	    fprintf(errstd,"Setting DELAY to 20 (2 seconds)\n");
+	    fprintf(errstd,"Warning: no PROMPT with SERIAL; setting DELAY to 20 (2 seconds)\n");
 	    delay = 20;
 	}
     }
@@ -866,11 +875,14 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 #else
 #define tick(x) ((x)*4000/2197)
 #endif
-    if (tick(delay) > 0xffff) die("Maximum delay is 59:59 (3599.5secs).");
-	else param2.delay = tick(delay);
+    delay =  delay==36000 ? 0xffff : tick(delay);
+    if (delay > 0xffff) die("Maximum delay is 59:59 (3599.5secs).");
+	else param2.delay = delay;
+
+    timeout =  timeout==36000 ? 0xfffe : tick(timeout);	/* -1 -> -1 ticks */
     if (timeout == -1) param2.timeout = 0xffff;
-    else if (tick(timeout) >= 0xffff) die("Maximum timeout is 59:59 (3599.5secs).");
-	else param2.timeout = tick(timeout);
+    else if (timeout >= 0xffff) die("Maximum timeout is 59:59 (3599.5secs).");
+	else param2.timeout = timeout;
 
 /* keytable & parameter area setup */
 
@@ -940,6 +952,7 @@ static int dev_number(char *dev)
     struct stat st;
 
     if (stat(dev,&st) >= 0) return st.st_rdev;
+    if (!isdigit(*dev)) die("Illegal 'root=' specification: %s", dev);
     if (verbose >= 1) 
 	printf("Warning: cannot 'stat' device \"%s\"; trying numerical conversion\n", dev);
     return to_number(dev);
@@ -1027,6 +1040,8 @@ static void bsect_common(IMAGE_DESCR *descr, int image)
     memset(options,0,SECTOR_SIZE);
     
 if (image) { /* long section specific to 'image=' */
+    char *append_local;
+    
     if ((cfg_get_flag(cf_kernel,"read-only") && cfg_get_flag(cf_kernel,
       "read-write")) || (cfg_get_flag(cf_options,"read-only") && cfg_get_flag(
       cf_options,"read-write")))
@@ -1038,12 +1053,16 @@ if (image) { /* long section specific to 'image=' */
       "read-write")) strcat(options,"rw ");
     if ((root = cfg_get_strg(cf_kernel,"root")) || (root = cfg_get_strg(
       cf_options,"root")))  {
-	if (strcasecmp(root,"current")) 
-	    sprintf(strchr(options,0),"root=%x ",dev_number(root));
-	else {
+	if (!strcasecmp(root,"current")) {
 	    if (stat("/",&st) < 0) pdie("stat /");
 	    sprintf(strchr(options,0),"root=%x ",(unsigned int) st.st_dev);
 	}
+	else if (strlen(root)>6 && !strncmp(root,"LABEL=",6)) {
+	    sprintf(strchr(options,0),"root=%s ", root);
+	}
+	else {
+	    sprintf(strchr(options,0),"root=%x ",dev_number(root));
+	}
       }	
     if ((ram_disk = cfg_get_strg(cf_kernel,"ramdisk")) || (ram_disk =
       cfg_get_strg(cf_options,"ramdisk")))
@@ -1071,8 +1090,19 @@ if (image) { /* long section specific to 'image=' */
 	strcat(options, " ");
     }
 #endif
-    if ((append = cfg_get_strg(cf_kernel,"append")) || (append =
-      cfg_get_strg(cf_options,"append"))) strcat(options,append);
+    append_local = cfg_get_strg(cf_options,"append");	/* global, actually */
+    if ((append = cfg_get_strg(cf_kernel,"append")) ||
+	(append = append_local)  )
+				strcat(strcat(options,append)," ");
+
+#if 1
+    append = append_local;	/* append == global append */
+    if ((append_local = cfg_get_strg(cf_kernel,"addappend"))) {
+	if (!append && !nowarn)
+	    fprintf(errstd, "Warning: ADDAPPEND used without global APPEND\n");
+	strcat(options,append_local);
+    }
+#endif
 
 } /* end of section specific to 'image=' */
 
@@ -1112,9 +1142,9 @@ if (image) { /* long section specific to 'image=' */
 	( (password = cfg_get_strg(cf_options,"password")) &&
 	  !cfg_get_flag(cf_all,"bypass")  ) ) {
 	if (!*password) {	/* null password triggers interaction */
-	    retrieve_crc((long*)descr->password_crc);
+	    retrieve_crc((int*)descr->password_crc);
 	} else {
-	    hash_password(password, (long*)descr->password_crc );
+	    hash_password(password, (int*)descr->password_crc );
 	}
 	descr->flags |= FLAG_PASSWORD;
     }
@@ -1171,7 +1201,7 @@ if (image) { /* long section specific to 'image=' */
     }
 #if 0
 #if 1
-    *(unsigned long *) descr->rd_size = 0; /* no RAM disk */
+    *(unsigned int *) descr->rd_size = 0; /* no RAM disk */
 #else
     descr->rd_size = 0; /* no RAM disk */
 #endif
@@ -1259,12 +1289,21 @@ void check_fallback(void)
     }
 }
 
+void check_unattended(void)
+{
+    if ( (descrs.d.descr[0].flags & (FLAG_PASSWORD + FLAG_RESTR) )
+							== FLAG_PASSWORD
+		&&  cfg_get_flag(cf_options,"unattended") )
+	die("Mandatory PASSWORD on default=\"%s\" defeats UNATTENDED",
+		descrs.d.descr[0].name);
+}
+
 
 void bsect_update(char *backup_file, int force_backup, int pass)
 {
     BOOT_SECTOR bsect_wr;
-    long temp;
-static long timestamp = 0;
+    int temp;
+static int timestamp = 0;
 
     if (pass>=0) {
 	temp = make_backup(backup_file, force_backup, &bsect_orig,
@@ -1281,7 +1320,7 @@ static long timestamp = 0;
 	menuparams.raid_dev_mask = raid_mask(menuparams.raid_offset);
 	memcpy(menuparams.serial_no, serial_no, sizeof(serial_no));
 	memcpy(table+256, &menuparams, sizeof(menuparams));
-	((long*)table)[SECTOR_SIZE/sizeof(long)-2] = crc32(table, SECTOR_SIZE-2*sizeof(long), CRC_POLY1);
+	((int*)table)[SECTOR_SIZE/sizeof(int)-2] = crc32(table, SECTOR_SIZE-2*sizeof(int), CRC_POLY1);
 	map_begin_section();
 	map_add_sector(table);
 #ifdef LCF_FIRST6
@@ -1337,7 +1376,7 @@ if (pass>=0) {
     if (verbose >= 3) {
 	printf("Failsafe check:  boot_dev_nr = 0x%04x 0x%04x\n", boot_dev_nr, has_partitions(boot_dev_nr));
 	/*** if (do_md_install) ***/ {
-	    printf("map==boot = %d    map s/n = %08lX\n",
+	    printf("map==boot = %d    map s/n = %08X\n",
 		!!(bsect_wr.par_1.prompt & FLAG_MAP_ON_BOOT),
 		bsect_wr.par_1.map_serial_no
 	    );
@@ -1510,7 +1549,7 @@ void bsect_uninstall(char *boot_dev,char *backup_file,int validate)
 }
 
 
-void bsect_raid_update(char *boot_dev, unsigned long raid_offset, 
+void bsect_raid_update(char *boot_dev, unsigned int raid_offset, 
 	char *backup_file, int force_backup, int pass, int mask)
 {
     BOOT_SECTOR bsect_save;
