@@ -1,7 +1,7 @@
 /* cfg.c  -  Configuration file parser */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -10,6 +10,7 @@ source directory.
 */
 
 
+#define _GNU_SOURCE
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -113,6 +114,7 @@ CONFIG cf_options[] = {
   { cft_flag, "static-bios-codes",	NULL,		NULL,NULL },
   { cft_flag, "suppress-boot-time-BIOS-data",	NULL,	NULL,NULL },
   { cft_strg, "timeout",	NULL,		NULL,NULL },
+  { cft_flag, "unattended",	NULL,		NULL,NULL },
   { cft_strg, "verbose",	NULL,		NULL,NULL },
   { cft_strg, "vga",		NULL,		NULL,NULL },
   { cft_strg, "vmdefault",	NULL,		NULL,NULL },
@@ -136,6 +138,7 @@ CONFIG cf_all[] = {
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_kernel[] = {
+  { cft_strg, "addappend",	NULL,		NULL,NULL },
   { cft_strg, "append",		NULL,		NULL,NULL },
   { cft_strg, "initrd",		NULL,		NULL,NULL },
   { cft_strg, "ramdisk",	NULL,		NULL,NULL },
