@@ -1,14 +1,14 @@
 #if 0
-/* common.h  -  Common data structures and functions. */
-/*
+  common.h  -  Common data structures and functions.
+
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
 source directory.
 
-*/
+
 #endif
 
 #ifndef COMMON_H
@@ -20,7 +20,7 @@ source directory.
 #include <asm/types.h>
 
 #ifndef PAGE_SIZE
-# define PAGE_SIZE 4096UL
+# define PAGE_SIZE 4096U
 #endif
 
 #ifdef O_ACCMODE
@@ -57,7 +57,7 @@ source directory.
 							block	0
 ;*/    char name[MAX_IMAGE_NAME+1];	/* image name, NUL terminated 
 						id_name:	.blkb	MAX_IMAGE_NAME_asm+1
-;*/    unsigned short password_crc[MAX_PW_CRC*(sizeof(long)/sizeof(short))];  /* 4 password CRC-32 values
+;*/    unsigned short password_crc[MAX_PW_CRC*(sizeof(int)/sizeof(short))];  /* 4 password CRC-32 values
 						id_password_crc:.blkb	MAX_PW_CRC_asm*4
 ;*/    unsigned short rd_size[2]; /* RAM disk size in sectors, 0 if none
 						id_rd_size:	.blkb	4		;don't change the order !!!
@@ -135,13 +135,13 @@ source directory.
 						par1_signature:	.blkb	4
 ;*/    unsigned short version;  /*
 						par1_version:	.blkb	2
-;*/    unsigned long map_stamp; /* timestamp for this installation (map creation)
+;*/    unsigned int map_stamp; /* timestamp for this installation (map creation)
 						par1_mapstamp:	.blkb	4
-;*/    unsigned long raid_offset; /* raid partition/partition offset
+;*/    unsigned int raid_offset; /* raid partition/partition offset
 						par1_raid_offset: .blkb	4
-;*/    unsigned long timestamp; /* timestamp for restoration
+;*/    unsigned int timestamp; /* timestamp for restoration
 						par1_timestamp:	.blkb	4
-;*/    unsigned long map_serial_no; /* volume serial no. / id containing the map file
+;*/    unsigned int map_serial_no; /* volume serial no. / id containing the map file
 						par1_map_serial_no:	.blkb	4
 ;*/    unsigned short prompt; /* FLAG_PROMPT=always, FLAG_RAID install
 						par1_prompt:	.blkb	2
@@ -162,7 +162,7 @@ source directory.
 						par2_signature:	.blkb	4
 ;*/    unsigned short version;	/*
 						par2_version:	.blkb	2
-;*/    unsigned long map_stamp;		/* time of creation of the map file
+;*/    unsigned int map_stamp;		/* time of creation of the map file
 						par2_mapstamp:	.blkb	4
 ;*/    unsigned short stage;	/*
 						par2_stage:	.blkb	2
@@ -257,9 +257,9 @@ source directory.
 						mt_mincol:	.blkw	1
 								.blkw	3
 
-;*/	unsigned long serial_no[MAX_BIOS_DEVICES];	/* Known device serial nos. 0x80 .. 0x8F
+;*/	unsigned int serial_no[MAX_BIOS_DEVICES];	/* Known device serial nos. 0x80 .. 0x8F
 						mt_serial_no:	.blkw	MAX_BIOS_DEVICES_asm*2
-;*/	unsigned long raid_offset[MAX_RAID_DEVICES];	/* RAID offsets for flagged devices
+;*/	unsigned int raid_offset[MAX_RAID_DEVICES];	/* RAID offsets for flagged devices
 						mt_raid_offset:	.blkw	MAX_RAID_DEVICES_asm*2
 ;*/	unsigned short raid_dev_mask;			/* 16 bit raid device mask flagging items in serial_no
 						mt_raid_dev_mask: .blkw	1
@@ -269,7 +269,7 @@ source directory.
 						mt_dflcmd:	.blkb	sa_size
 ;*/	SECTOR_ADDR mt_descr[MAX_DESCR_SECS];	/* descriptor disk addresses
 						mt_descr:	.blkb	sa_size*MAX_DESCR_SECS_asm
-;*/	char unused[150-MAX_BIOS_DEVICES*sizeof(long)-(MAX_RAID_DEVICES)*sizeof(long)-MAX_DESCR_SECS*sizeof(SECTOR_ADDR)];		/* spacer
+;*/	char unused[150-MAX_BIOS_DEVICES*sizeof(int)-(MAX_RAID_DEVICES)*sizeof(int)-MAX_DESCR_SECS*sizeof(SECTOR_ADDR)];		/* spacer
 						mt_unused:	.blkb	150-sa_size*MAX_DESCR_SECS_asm-4*MAX_BIOS_DEVICES_asm-4*MAX_RAID_DEVICES_asm
 ;*/	short checksum[2];		/* checksum longword
 						mt_cksum:	.blkw	2
@@ -288,7 +288,7 @@ source directory.
 typedef struct {
     unsigned char bootcode[MAX_BOOT_SIZE];
     unsigned short mbz;		/* must be zero */
-    long  volume_id;
+    int  volume_id;
     unsigned short marker;	/* may be zero */
     unsigned char part[PART_TABLE_SIZE];
     unsigned short boot_ind;	/* 0xAA55 */
@@ -309,8 +309,8 @@ typedef union {
     } d;
     unsigned char sector[SECTOR_SIZE*MAX_DESCR_SECS];
     struct {
-    	unsigned long sector[SECTOR_SIZE/4*MAX_DESCR_SECS - 1];
-    	unsigned long checksum;
+    	unsigned int sector[SECTOR_SIZE/4*MAX_DESCR_SECS - 1];
+    	unsigned int checksum;
     } l;
 } DESCR_SECTORS;
 
@@ -324,7 +324,7 @@ typedef struct {
 #pragma pack (2)
 typedef struct {
     unsigned char device, flags;
-    unsigned long sector;
+    unsigned int sector;
 } SECTOR_ADDR6;
 #pragma pack ()
 #endif
@@ -345,14 +345,14 @@ typedef struct {
     unsigned char loader;	/* 16: loader type */
     unsigned char flags;	/* 17: loader flags */
     unsigned short a;		/* 18: more LOADLIN hacks */
-    unsigned long start;	/* 20: kernel start, filled in by loader */
-    unsigned long ramdisk;	/* 24: RAM disk start address */
-    unsigned long ramdisk_size;	/* 28: RAM disk size */
+    unsigned int start;	/* 20: kernel start, filled in by loader */
+    unsigned int ramdisk;	/* 24: RAM disk start address */
+    unsigned int ramdisk_size;	/* 28: RAM disk size */
     unsigned short b,c;		/* 32: bzImage hacks */
     unsigned short heap_end_ptr;/* 36: 2.01 end of free area after setup code */
     unsigned char d;		/* 38: padding */
-    unsigned long cmd_line_ptr; /* 40: 2.02 address32 of command line */
-    unsigned long ramdisk_max;	/* 44: 2.03 address32 of highest mem. for ramdisk */
+    unsigned int cmd_line_ptr; /* 40: 2.02 address32 of command line */
+    unsigned int ramdisk_max;	/* 44: 2.03 address32 of highest mem. for ramdisk */
 } SETUP_HDR;
 
 #define alloc_t(t) ((t *) alloc(sizeof(t)))
@@ -371,9 +371,9 @@ extern DL_BIOS bios_passes_dl;
 extern int boot_dev_nr,raid_flags,do_md_install,ndisk,zflag,eflag;
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
 extern int curr_drv_map;
-extern unsigned long prt_map[PRTMAP_SIZE+1];
+extern unsigned int prt_map[PRTMAP_SIZE+1];
 extern int curr_prt_map, config_read;
-extern unsigned long serial_no[MAX_BIOS_DEVICES];
+extern unsigned int serial_no[MAX_BIOS_DEVICES];
 extern char *config_file;
 extern FILE *errstd;
 extern FILE *pp_fd;
@@ -381,8 +381,8 @@ extern char *identify;	/* in identify.c */
 
 #define crc(a,b) (~crc32((a),(b),CRC_POLY1))
 #define brev(x)  \
- { register unsigned short u1,u2; register unsigned long u=(x); u1=u; u2=u>>16; \
- u1=(u1>>8)|(u1<<8); u2=(u2>>8)|(u2<<8); x=(unsigned long)u1<<16|u2; }
+ { register unsigned short u1,u2; register unsigned int u=(x); u1=u; u2=u>>16; \
+ u1=(u1>>8)|(u1<<8); u2=(u2>>8)|(u2<<8); x=(unsigned int)u1<<16|u2; }
 #define cc(x) crc(x.data,x.size)
 #define comma (cc(First)|cc(Second)|cc(Third)|cc(Bitmap)|cc(Chain)|cc(Mbr)|cc(Mbr2))
 
@@ -411,6 +411,12 @@ int to_number(char *num);
 /* Converts a string to a number. Dies if the number is invalid. */
 
 
+int timer_number(char *num);
+/* Converts a string to a number.  Allows suffix of 't', 's', 'm', 'h'
+for Tenths, Seconds, Minutes, Hours. Dies if number is invalid for 
+"timeout" or "delay" */
+
+
 void check_version(BOOT_SECTOR *sect, int stage);
 /* Verify that a boot sector has the correct version number. */
 
@@ -421,13 +427,13 @@ int stat_equal(struct stat *a, struct stat *b);
 
 
 
-unsigned long crc32partial(unsigned char *cp, int nsize,
-			unsigned long polynomial, unsigned long *accum);
+unsigned int crc32partial(unsigned char *cp, int nsize,
+			unsigned int polynomial, unsigned int *accum);
 /* accumulate a partial CRC-32 */
 
 
 
-unsigned long crc32 (unsigned char *cp, int nsize, unsigned long polynomial);
+unsigned int crc32 (unsigned char *cp, int nsize, unsigned int polynomial);
 /* calculate a CRC-32 polynomial */
 
 
