@@ -1,7 +1,7 @@
 /* device.c  -  Device access */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -55,7 +55,8 @@ static CACHE_ENTRY *cache = NULL;
 int yesno(char *prompt, int expect)
 {
     char *line, *pr2;
-    int i, n;
+    int i;
+    size_t n;
     int ret;
     if (expect) pr2 = "[Y/n]";
     else pr2 = "[N/y]";
@@ -397,13 +398,13 @@ static int dev_listed(unsigned short dev)
 /* make a backup, returning the timestamp of the backup file */
 /* 0 if no timestamp returned, and no backup file created */
 
-long make_backup(char *backup_file, int force_backup, BOOT_SECTOR *bsect,
+int make_backup(char *backup_file, int force_backup, BOOT_SECTOR *bsect,
 	int device, char *id)
 {
     struct stat st;
     char temp_name[PATH_MAX+1];
     int bck_file;
-    long timestamp=0;
+    int timestamp=0;
     char *filename = "boot";
 
     char *cp = NULL;
@@ -503,7 +504,7 @@ long make_backup(char *backup_file, int force_backup, BOOT_SECTOR *bsect,
 
 
 
-int serial_valid(unsigned long serial, int disk_bios)
+int serial_valid(unsigned int serial, int disk_bios)
 {
 #if 1
     return (serial != 0);
@@ -518,7 +519,7 @@ int serial_valid(unsigned long serial, int disk_bios)
 		FFFFFFFF
 BUT: any Volume ID (except 0 or -1) is valid on drive C:
  - - - - - - - - - - - - - - - - - - - - - - - - - - */
-    unsigned long temp;
+    unsigned int temp;
     
     temp = serial & 0xFF;
     temp |= temp << 8;
@@ -535,18 +536,19 @@ static int inited = 0;
    
     if (!inited) {
 	struct stat st;
-	int fd;
-#define RANDOM "/dev/random"	
+	int fd, random;
+#define RANDOM "/dev/urandom"	
 
+	inited = time(NULL);
 	if ( stat(RANDOM, &st)==0 && S_ISCHR(st.st_mode)
 	    && (fd = open(RANDOM, O_RDONLY)) > 0
-	    && read(fd, &inited, sizeof(inited)) == sizeof(inited) )  {
+	    && read(fd, &random, sizeof(random)) == sizeof(random) )  {
 #if BETA_TEST
 if(verbose>=5) printf("Using " RANDOM " for seeding random number generator\n");	    
 #endif
 		close(fd);
+		inited ^= random;
 	}
-	else inited = time(NULL);
 
 	srand(inited);
     }
@@ -561,7 +563,7 @@ if(verbose>=5) printf("Using " RANDOM " for seeding random number generator\n");
 
 
 static int inited = 0;
-unsigned long serial_no[MAX_BIOS_DEVICES];
+unsigned int serial_no[MAX_BIOS_DEVICES];
 static int device_code[MAX_BIOS_DEVICES];
 
 /*  register references to various bios devices
@@ -570,12 +572,12 @@ static int device_code[MAX_BIOS_DEVICES];
  *	0 = no volume ID
  *	-1 = error
  */
-unsigned long register_bios(int bios, int device)
+unsigned int register_bios(int bios, int device)
 {
     int i, fd, valid, disk_bios;
     DEVICE dev;
     BOOT_SECTOR buff;
-    unsigned long serial = -1;
+    unsigned int serial = -1;
     
 #if 0
     if (!inited) {
@@ -606,7 +608,7 @@ unsigned long register_bios(int bios, int device)
 		die("master boot record seek %04X: %s", device, strerror(errno));
 	    if (read(fd, (char*)&buff, SECTOR_SIZE)!=SECTOR_SIZE)
 		die("read master boot record %04X: %s", device, strerror(errno));
-	    serial = *(long*)&buff.sector[PART_TABLE_OFFSET-6];
+	    serial = *(int*)&buff.sector[PART_TABLE_OFFSET-6];
 	    valid = serial_valid(serial, disk_bios);
 	    if ((!valid || (VERSION_MINOR>=50)) && !test)
 		    make_backup(NULL, 0, &buff, device,
@@ -617,7 +619,7 @@ unsigned long register_bios(int bios, int device)
 		for (; i<5 && !serial_valid(serial, disk_bios); i++) serial = rand();
 		if (!serial_valid(serial, disk_bios)) die("Volume ID generation error");
 
-		*(long*)&buff.sector[PART_TABLE_OFFSET-6] = serial;
+		*(int*)&buff.sector[PART_TABLE_OFFSET-6] = serial;
 		if (*(short*)&buff.sector[PART_TABLE_OFFSET - 2] == 0)
 		    *(short*)&buff.sector[PART_TABLE_OFFSET - 2] = MAGIC_SERIAL;
 		if (verbose)
@@ -692,7 +694,7 @@ static int volid_get_set(int device, int vol_in, int option)
     BOOT_SECTOR buf;
     DEVICE dev;
     int fd;
-    long temp;
+    int temp;
     unsigned short word;
     
     if (!has_partitions(device) && (device & P_MASK(device)) )
@@ -843,16 +845,17 @@ static int winnt_check(struct VolumeMgmt *vm, int fatal)
 
 int pf_hard_disk_scan(void)
 {
-    struct VolumeMgmt vm [MAX_BIOS_DEVICES];
+    struct VolumeMgmt vm [MAX_DEVICES];
 static int warned = 0, called = 0;
     char *line, *next, *name;
-    int n, major, minor, i, ipart;
-    int dev, device, lastdevice, mask, bios;
+    int major, minor, i, ipart;
+    int dev, device, mask, bios;
+    size_t n;
     DEVICE Dev;
     DT_ENTRY *walk;
     struct stat st;
     int duplicate = 0, invalid = 0, ret = 0, ntcaution = 0;
-    int codes = 0;
+    long codes = 0L;
 
 /* called from  raid_setup  &  from  geo_open */
 /* allow only 1 call */
@@ -869,7 +872,7 @@ static int warned = 0, called = 0;
 	return 1;
     }
 
-    lastdevice = n = 0;
+    n = 0;
     line = NULL;
     while (!feof(pp_fd)) {
     	if (line) {
@@ -949,7 +952,7 @@ static int warned = 0, called = 0;
 #endif
 	if (mask && ipart>0) {
 	    int found;
-	    long serial;
+	    int serial;
 	   
 	    for (found=i=0; i<ndevs && !found; i++) {
 		if (dev==vm[i].device) found = i+1;
@@ -964,7 +967,17 @@ static int warned = 0, called = 0;
 		    serial = volid_get_set(dev, new_serial(dev), ID_SET);
 		}
 #endif
-
+#if BETA_TEST
+		if (verbose>=3) printf("**ndevs=%d\n", ndevs);
+#endif
+		if (ndevs>=MAX_DEVICES) {
+		    die("More than %d hard disks are listed in '" PARTITIONS "'.\n"
+			"    Disks beyond the %dth must be marked:\n"
+			"        disk=/dev/XXXX  inaccessible\n"
+			"    in the configuration file (/etc/lilo.conf).\n"
+			, MAX_DEVICES, MAX_DEVICES);
+		}
+		else
 		{
 		    GEOMETRY geo;
 		    
@@ -1028,7 +1041,7 @@ static int warned = 0, called = 0;
 			invalid++;
 		    }
 		    found = ++ndevs;
-		} /* if (open, llseek, read ) */
+		} /* if (open, lseek, read ) */
 				
 	    } /* if (!found)  */
 
@@ -1101,7 +1114,7 @@ static int warned = 0, called = 0;
 			walk->next = disktab;
 			vm[j].dt = disktab = walk;
 #if BETA_TEST
-			if (verbose >= 4) printf("Allocated DT entry for device %04X  ptr=%08x\n", vm[j].device, (int)walk);
+			if (verbose >= 4) printf("Allocated DT entry for device %04X  ptr=%08lx\n", vm[j].device, (long)walk);
 #endif
 		    }
 
@@ -1201,7 +1214,7 @@ static int warned = 0, called = 0;
 
 
 
-    if (verbose>=2) printf("device codes (user assigned pf) = %X\n", codes);
+    if (verbose>=2) printf("device codes (user assigned pf) = %lX\n", codes);
 
 /* mark those BIOS codes that are already used in the disk=/bios= table */
 
@@ -1231,7 +1244,7 @@ static int warned = 0, called = 0;
 	}
     }
 
-    if (verbose>=2) printf("device codes (user assigned) = %X\n", codes);
+    if (verbose>=2) printf("device codes (user assigned) = %lX\n", codes);
 
     for (i=0; i<ndevs; i++) {
 	bios = vm[i].bios.probe;
@@ -1244,41 +1257,61 @@ static int warned = 0, called = 0;
 	}
     }
 
-    if (verbose>=2) printf("device codes (BIOS assigned) = %X\n", codes);
+    if (verbose>=2) printf("device codes (BIOS assigned) = %lX\n", codes);
 
     for (bios=i=0; i<ndevs; i++) {
 	int j;
 	
+	
 	if (vm[i].bios.actual < 0x80) {
 	    while ( codes & (1L<<bios) ) bios++;
-	    codes |= 1L<<bios;
-	    vm[i].bios.actual = 0x80+bios;
-	    if (verbose>=3) printf("Filling in '%s' = 0x%02X\n", vm[i].name, bios+0x80);
+	    if (bios < MAX_BIOS_DEVICES) {
+		codes |= 1L<<bios;
+		vm[i].bios.actual = 0x80+bios;
+		if (verbose>=3) printf("Filling in '%s' = 0x%02X\n", vm[i].name, bios+0x80);
+	    }
+	    else vm[i].bios.actual = -1;
 	}
 	if (!vm[i].dt) {
 	    walk = alloc_t(DT_ENTRY);
 	    walk->device = vm[i].device;
-	    walk->cylinders = walk->heads = walk->sectors = walk->start = -1;
+	    walk->cylinders = walk->bios = walk->heads = walk->sectors = walk->start = -1;
 	    walk->next = disktab;
 	    vm[i].dt = disktab = walk;
 #if BETA_TEST
-	    if (verbose >= 4) printf("Allocated DT entry for device %04X  ptr=%08x\n", vm[i].device, (int)walk);
+	    if (verbose >= 4) printf("Allocated DT entry for device %04X  ptr=%08lx\n", vm[i].device, (long)walk);
 #endif
 	}
 	j = vm[i].dt->bios = vm[i].bios.actual;
 	j &= 0x7F;
-	
-	serial_no[j] = vm[i].vol_id.kernel;
-	device_code[j] = vm[i].device;
+
+	if (j < MAX_BIOS_DEVICES) {	
+	    serial_no[j] = vm[i].vol_id.kernel;
+	    device_code[j] = vm[i].device;
 #if BETA_TEST
-	if (verbose >= 5) {
-	    printf("Generated: %02X  %04X  %08X\n", j+0x80, device_code[j], (int)serial_no[j]);
-	}
+	    if (verbose >= 5) {
+		printf("Generated: %02X  %04X  %08X\n", j+0x80, device_code[j], (int)serial_no[j]);
+	    }
 #endif
+	}
+	else {
+	    vm[i].dt->heads = 0;	/* mark inaccessible */
+	    if (!nowarn) {
+		static int i=0;
+		if (!(i++)) fprintf(errstd,
+		    "Warning: Internal implementation restriction. Boot may occur from the first\n"
+		    "    %d disks only. Disks beyond the %dth will be flagged INACCESSIBLE.\n"
+		    , MAX_BIOS_DEVICES, MAX_BIOS_DEVICES);
+		fprintf(errstd,
+		"Warning: 'disk=%s  inaccessible' is being assumed.  (%04X)\n",
+		vm[i].name, vm[i].device);
+	    }
+	}
+	
 	inited = 1;
     }
 
-    if (verbose>=2) printf("device codes (canonical) = %X\n", codes);
+    if (verbose>=2) printf("device codes (canonical) = %lX\n", codes);
 
     for (bios=8*sizeof(codes)-1; !(codes&(1L<<bios)) && bios>=0; ) bios--;
 
