@@ -1,6 +1,6 @@
 /* test4.c */
 /*
- Copyright 2001-2003 John Coffman.
+ Copyright 2001-2004 John Coffman.
  All rights reserved.
 
  Licensed under the terms contained in the file 'COPYING' in the LILO
@@ -10,7 +10,7 @@
 #include <bios.h>
 #include "../bdata.h"
 
-#define DISK_VERSION "2.2.2"
+#define DISK_VERSION "2.2.4"
 
 #if __MSDOS__==0
 #define putch bios_putc
@@ -226,7 +226,7 @@ void banner(char *version)
 {
 	printf(	"\n\n\n"
 ">>>> Disk Detection and Parameter Display <<<<\n\n\n"
-"Version %s, Copyright (C) 1999-2003  John Coffman <johninsd@san.rr.com>\n"
+"Version %s, Copyright (C) 1999-2004  John Coffman <johninsd@san.rr.com>\n"
 "Portions Copyright (C) 1996-2001 Robert de Bath, used with permission\n"
 "Re-use and redistribution rights set forth in the file \"COPYING\".\n\n",
 	 version);
@@ -901,6 +901,8 @@ static char *dt[] = {	"5.25\", 40 track, 360K",
 			"Call With:\n    ", dev);
 	reg.x.ax = 0x0800;
 	reg.x.dx = dev;
+	reg.x.di = 0x4321;
+	reg.x.bx = 0x1234;
 	segread(&sreg);
 	sreg.es = 0;
 	print_sregs(&sreg);
@@ -923,6 +925,12 @@ static char *dt[] = {	"5.25\", 40 track, 360K",
 		    printf("Parameter table at %04x:%04x\n",
 					sreg.es, oreg.x.di);
 		}
+		else {
+		    if (oreg.x.bx != reg.x.bx)
+			printf("Error:  Hard disk BIOS should not touch BX\n");
+		    if (sreg.es != 0  ||  oreg.x.di != reg.x.di)
+			printf("Error:  Hard disk BIOS should not touch ES:DI\n");
+		}
 		s = (oreg.h.cl & 0x3F);		/* sectors 1..63 */
 		h = (int)oreg.h.dh + 1;		/* heads 0..254 */
 		c = (((oreg.h.cl & 0xC0)<<2) | oreg.h.ch) + 1;
