@@ -1,7 +1,7 @@
 /* geometry.c  -  Device and file geometry computation */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -10,6 +10,7 @@ source directory.
 */
 /* Patched for linux-2.4.0 - Glibc-2.2 by Sergey Ostrovsky 11/16/2000 */
 
+#define _GNU_SOURCE
 #include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
@@ -67,10 +68,10 @@ struct lv_bmap {
 };
 
 #ifndef LV_BMAP
-#define LV_BMAP				_IOWR(0xfe, 0x30, 1)
+#define LV_BMAP				_IOWR(0xfe, 0x30, char)
 #endif
 #ifndef LVM_GET_IOP_VERSION
-#define LVM_GET_IOP_VERSION		_IOR(0xfe, 0x98, 1)
+#define LVM_GET_IOP_VERSION		_IOR(0xfe, 0x98, char)
 #endif
 #endif
 
@@ -197,11 +198,18 @@ int has_partitions_beta(dev_t dev)
       (major >= MAJOR_I2O && major <= MAJOR_I2O+7) ||
       (major >= MAJOR_SMART2 && major <= MAJOR_SMART2+7) ||
       (major >= MAJOR_CISS && major <= MAJOR_CISS+7) ||
-      major == MAJOR_FTL || major == MAJOR_NFTL || major == MAJOR_DOC
+      major == MAJOR_FTL || major == MAJOR_NFTL || major == MAJOR_DOC ||
+      (major >= MAJOR_SD9 && major <= MAJOR_SD16) ||
+      major == MAJOR_EMD
         ) return 0xFFF0;	/* 4 bit partition mask */
 
+    if ( major == MAJOR_SATA || major == MAJOR_SATA2
+        )  return 0xFFE0;	/* 5 bit partition mask */
+
+
     if ( major == MAJOR_IBM_iSER ||
-      (major >= MAJOR_DAC960 && major <= MAJOR_DAC960+7)
+      (major >= MAJOR_DAC960 && major <= MAJOR_DAC960+7) ||
+      (major >= MAJOR_DAC960_8 && major <= MAJOR_DAC960_8+7)
         )  return 0xFFF8;	/* 3 bit partition mask */
 
     return 0;
@@ -228,9 +236,9 @@ unsigned char max_partno[256] = {
 /*
   0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
 */
-  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   /*  8x   */
-  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+ 15, 15, 15, 15, 15, 15, 15, 15,  7,  7,  7,  7,  7,  7,  7,  7,   /*  8x   */
+  0,  0,  0,  0,  0,  0,  0,  0,  0, 15,  0,  0,  0,  0,  0,  0,
+ 31, 31,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
 
   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,   /*  Cx   */
@@ -283,7 +291,7 @@ void do_disk(void)
 	int i = to_number(maxpart);
 	if (max_partno[major] && max_partno[major]!=i) die("Cannot alter 'max-partitions' for known disk  %s", disk);
 	max_partno[major] = i;
-	if (i!=7 && i!=15 && i!=63) die("disk=%s:  illegal value for max-partitions(%d)", disk, i);
+	if (i!=7 && i!=15 && i!=31 && i!=63) die("disk=%s:  illegal value for max-partitions(%d)", disk, i);
     }
     entry->bios = bios ? to_number(bios) : -1;
     if (!sectors && !heads) entry->sectors = entry->heads = -1;
@@ -515,7 +523,7 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
     get_all = all;
 #endif
     if (!MAJOR(device))
-	die("Trying to map files from unnamed device 0x%04x (NFS ?)",device);
+	die("Trying to map files from unnamed device 0x%04x (NFS/RAID mirror down ?)",device);
     if (device == MAJMIN_RAM)
 	die("Trying to map files from your RAM disk. "
 	  "Please check -r option or ROOT environment variable.");
@@ -890,7 +898,7 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	for (walk = disktab; walk; walk = walk->next)
 	    if (walk->device == (device & D_MASK(device))) break;
 #if BETA_TEST
-    if (verbose>=5) printf("walk=%08x\n", (int)walk);
+    if (verbose>=5) printf("walk=%08lx\n", (long)walk);
 #endif
 
 #if 1
