@@ -10,6 +10,7 @@ source directory.
 */
 
 
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -20,6 +21,7 @@ source directory.
 #include <time.h>
 #include <limits.h>
 #include <sys/stat.h>
+#include <dirent.h>
 /*#include <asm/page.h>*/
 
 #include "config.h"
@@ -149,6 +151,7 @@ static void show_images(char *map_file)
 	    printf("  Command-line timeout: %d.%d seconds\n",tsecs/10,
 	      tsecs % 10);
 	}
+	printf("  %snattended booting\n", param2.flag2&FLAG2_UNATTENDED ? "U" : "No u");
 	if (boot.par_1.prompt & FLAG_PROMPT) printf("  Always enter boot prompt\n");
 	else printf("  Enter boot prompt only on demand\n");
 	printf("  Boot-time BIOS data%s saved\n",
@@ -173,7 +176,7 @@ static void show_images(char *map_file)
 	    printf("  No default boot command line\n");
 	else printf("  Default boot command line: \"%s\"\n",buffer+2);
 	if (verbose>=3) {
-	    printf("Serial numbers %08lX\n", menu.serial_no[0]);
+	    printf("Serial numbers %08X\n", menu.serial_no[0]);
 	}
 	printf("Images:\n");
     }
@@ -251,10 +254,10 @@ static void show_images(char *map_file)
 		    if (!(flags & FLAG_LOADHI))
 			printf("    Kernel is loaded \"low\"\n");
 		    else printf("    Kernel is loaded \"high\"\n");
-		    if (!*(unsigned long *) descrs.d.descr[image].rd_size)
+		    if (!*(unsigned int *) descrs.d.descr[image].rd_size)
 			printf("    No initial RAM disk\n");
-		    else printf("    Initial RAM disk is %ld bytes\n",
-			  *(unsigned long *) descrs.d.descr[image].rd_size);
+		    else printf("    Initial RAM disk is %d bytes\n",
+			  *(unsigned int *) descrs.d.descr[image].rd_size);
 		    if (flags & FLAG_TOOBIG)
 			printf("       and is too big to fit between 4M-15M\n");
 		}
@@ -349,7 +352,7 @@ static char *bitmap_file;
     int more,version,uninstall,validate,activate,instmbr,geom;
     struct stat st;
     int fd, temp, tell_early=0;
-    long raid_offset;
+    int raid_offset;
 
     errstd = stderr;
 #if VERSION_MINOR>=50
@@ -463,7 +466,11 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		passw = 1;	/* force re-gen of password file */
 		break;
 	    case 'P':
-		if (!strcasecmp(param,"fix"))
+		if ((act1=strchr(param,'='))) {
+		    *act1++ = 0;	/* null terminate */
+		    cfg_set(cf_options,param,act1,NULL);
+		}
+		else if (!strcasecmp(param,"fix"))
 		    cfg_set(cf_options,"fix-table",NULL,NULL);
 		else if (!strcasecmp(param,"ignore"))
 		    cfg_set(cf_options,"ignore-table",NULL,NULL);
@@ -553,8 +560,8 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		"-DBOOT_PARAMS_1=%d -DBOOT_PARAMS_2=%d  PATH_MAX=%d"
 		  "\n"
 		  ,
-		  sizeof(BOOT_PARAMS_1),
-		  sizeof(BOOT_PARAMS_2),
+		  (int)sizeof(BOOT_PARAMS_1),
+		  (int)sizeof(BOOT_PARAMS_2),
 		  PATH_MAX
 		     );
 }
@@ -566,61 +573,61 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	RGB2 pal2[16];
 	
 	printf("\nBMFH=%d BMH=%d BMH2=%d RGB=%d(%d) RGB2=%d(%d) LH=%d\n",
-	sizeof(BITMAPFILEHEADER),
-	sizeof(BITMAPHEADER),
-	sizeof(BITMAPHEADER2),
-	sizeof(RGB),
-	sizeof(pal),
-	sizeof(RGB2),
-	sizeof(pal2),
-	sizeof(BITMAPLILOHEADER) );
+	(int)sizeof(BITMAPFILEHEADER),
+	(int)sizeof(BITMAPHEADER),
+	(int)sizeof(BITMAPHEADER2),
+	(int)sizeof(RGB),
+	(int)sizeof(pal),
+	(int)sizeof(RGB2),
+	(int)sizeof(pal2),
+	(int)sizeof(BITMAPLILOHEADER) );
 	printf("\nBPB=%d BP_DOS=%d\n",
-		sizeof(BIOS_PARAMETER_BLOCK),
-		sizeof(BOOT_PARAMS_DOS)
+		(int)sizeof(BIOS_PARAMETER_BLOCK),
+		(int)sizeof(BOOT_PARAMS_DOS)
 		  );
 }
 #ifdef LCF_FIRST6
-	printf("\nSECTOR_ADDR6=%d\n", sizeof(SECTOR_ADDR6));
+	printf("\nSECTOR_ADDR6=%d\n", (int)sizeof(SECTOR_ADDR6));
 #endif
 
 #endif
 {
-  unsigned long j;
+  unsigned int j;
   if (verbose>=5) {
     if (crc(First.data,First.size)) {
 	j = crc(First.data,First.size-4);
 	brev(j);
-	printf("1=0x%lx\n", j);
+	printf("1=0x%x\n", j);
     }
     if (crc(Second.data,Second.size)) {
 	j = crc(Second.data,Second.size-4);
 	brev(j);
-	printf("2=0x%lx\n", j);
+	printf("2=0x%x\n", j);
     }
     if (crc(Third.data,Third.size)) {
 	j = crc(Third.data,Third.size-4);
 	brev(j);
-	printf("3=0x%lx\n", j);
+	printf("3=0x%x\n", j);
     }
     if (crc(Bitmap.data,Bitmap.size)) {
 	j = crc(Bitmap.data,Bitmap.size-4);
 	brev(j);
-	printf("B=0x%lx\n", j);
+	printf("B=0x%x\n", j);
     }
     if (crc(Chain.data,Chain.size)) {
 	j = crc(Chain.data,Chain.size-4);
 	brev(j);
-	printf("C=0x%lx\n", j);
+	printf("C=0x%x\n", j);
     }
     if (crc(Mbr.data,Mbr.size)) {
 	j = crc(Mbr.data,Mbr.size-4);
 	brev(j);
-	printf("M=0x%lx\n", j);
+	printf("M=0x%x\n", j);
     }
     if (crc(Mbr2.data,Mbr2.size)) {
 	j = crc(Mbr2.data,Mbr2.size-4);
 	brev(j);
-	printf("N=0x%lx\n\n", j);
+	printf("N=0x%x\n\n", j);
     }
   }
 }
@@ -649,13 +656,25 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	BOOT_SECTOR b;
 	dev = 0xFFF0;
 	i = dev;
-	printf("dev_t is %ssigned size=%d  i=%08X\n", i==0xFFF0 ? "un" : "", sizeof(dev_t), i);
-	printf("size of BOOT_VOLID = %d   BOOT_SECTOR = %d\n", sizeof(x), sizeof(b));
-	printf("Size of MENUTABLE = %d\n", sizeof(MENUTABLE));
+	printf("dev_t is %ssigned size=%d  i=%08X\n", i==0xFFF0 ? "un" : "", (int)sizeof(dev_t), i);
+	printf("size of BOOT_VOLID = %d   BOOT_SECTOR = %d\n", (int)sizeof(x), (int)sizeof(b));
+	printf("Size of MENUTABLE = %d\n", (int)sizeof(MENUTABLE));
 }
 #endif
 		printf("\nCFLAGS = " CFLAGS "\n");
-		printf("\nMAX_IMAGES = %d\n\n", MAX_IMAGES);
+		printf("\nglibc version %d.%d\n", __GLIBC__, __GLIBC_MINOR__);
+		printf("Kernel Headers included from  %d.%d.%d\n",
+			LINUX_VERSION_CODE>>16,
+			LINUX_VERSION_CODE>>8 & 255,
+			LINUX_VERSION_CODE & 255);
+		printf("\nMAX_IMAGES = %d\t\tc=%d, s=%d, i=%d, "
+				"l=%d, ll=%d, f=%d, d=%d, ld=%d\n\n",
+			MAX_IMAGES, (int)sizeof(char),
+			(int)sizeof(short), (int)sizeof(int),
+			(int)sizeof(long), (int)sizeof(long long),
+			(int)sizeof(float), (int)sizeof(double),
+			(int)sizeof(long double)
+			);
 		exit(0);
 	    case 'z':
 		zflag++;	/* force zero of MBR 8-byte area */
@@ -672,7 +691,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
     if (new_root && *new_root) {
 	pp_fd = fopen(PARTITIONS, "r");
 	if (chroot(new_root) < 0) die("chroot %s: %s",new_root,strerror(errno));
-	if (chdir("/dev") < 0 && nowarn==0)
+	if (chdir("/dev") < 0  &&  nowarn < 1 )
 	    fprintf(errstd, "Warning: root at %s has no /dev directory\n", new_root);
 	if (chdir("/") < 0) die("chdir /: %s",strerror(errno));
     }
@@ -689,8 +708,19 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
 	       "Development beyond version 21 Copyright (C) 1999-2004 John Coffman\n"
 	       );
-        if (verbose>0) printf("Released %s%s and compiled at %s on %s%s\n",
-	        VERSION_DATE, comma ? "," : "", __TIME__, __DATE__, semi);
+        if (verbose>0) {
+#if VERSION_MINOR>=50
+#include <sys/utsname.h>
+	    struct utsname buf;
+#endif
+            printf("Released %s%s and compiled at %s on %s%s\n",
+		VERSION_DATE, comma ? "," : "", __TIME__, __DATE__, semi);
+#if VERSION_MINOR>=50
+	    if (uname(&buf)==0) {
+		printf("Running %s kernel %s\n", buf.sysname, buf.release);
+	    }
+#endif
+	}
         printf("\n");
         if (version) return 0;
     }
@@ -766,28 +796,27 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	"  use LINEAR only if you are aware of its limitations.\n\n");
     
     if (identify) identify_image(identify,ident_opt);
-    
+
+    if (uninstall)
+	bsect_uninstall(uninst_dev ? uninst_dev : cfg_get_strg(cf_options,
+	      "boot"),cfg_get_strg(cf_options,"backup"),validate);
+
+    if (reboot_arg) {
+	map_patch_first(cfg_get_strg(cf_options,"map") ? cfg_get_strg(
+	      cf_options,"map") : MAP_FILE, reboot_arg);
+	exit(0);
+    }
+
     if ( (param = cfg_get_strg(cf_options,"bios-passes-dl")) ) {
 	if (strchr("YyTt1", *param)) bios_passes_dl = DL_GOOD;
 	if (strchr("NnFf0", *param)) bios_passes_dl = DL_BAD;
     }
     if (bios_passes_dl == DL_NOT_SET) 	check_bios();	/* in probe.c */
 
-    
-
-    if (uninstall)
-	bsect_uninstall(uninst_dev ? uninst_dev : cfg_get_strg(cf_options,
-	      "boot"),cfg_get_strg(cf_options,"backup"),validate);
     if (!nowarn && compact && (linear || lba32))
 	fprintf(errstd,"Warning: COMPACT may conflict with %s on some "
 		"systems\n", lba32 ? "LBA32" : "LINEAR");
-    if (reboot_arg) {
-	map_patch_first(cfg_get_strg(cf_options,"map") ? cfg_get_strg(
-	      cf_options,"map") : MAP_FILE, reboot_arg);
-/*	sync();	*/
-	exit(0);
-    }
-    if (argc) usage(name);
+
     geo_init(cfg_get_strg(cf_options,"disktab"));
     if (query)
 	show_images(!cfg_get_strg(cf_options,"map") ? MAP_FILE :
@@ -801,7 +830,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 /* test for a RAID installation */
 	raid_offset = raid_setup();
 	if (verbose >= 2) {
-	    printf("raid_setup returns offset = %08lX  ndisk = %d\n", raid_offset, ndisk);
+	    printf("raid_setup returns offset = %08X  ndisk = %d\n", raid_offset, ndisk);
 	    dump_serial_nos();    
 	}
 
@@ -814,9 +843,9 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 			cfg_get_strg(cf_options,"map") : MAP_FILE,
 		cfg_get_strg(cf_options,"install"),
 		cfg_get_strg(cf_options,"delay") ?
-			to_number(cfg_get_strg(cf_options,"delay")) : 0,
+			timer_number(cfg_get_strg(cf_options,"delay")) : 0,
 		cfg_get_strg(cf_options,"timeout") ?
-			to_number(cfg_get_strg(cf_options,"timeout")) : -1,
+			timer_number(cfg_get_strg(cf_options,"timeout")) : -1,
 		raid_offset );
 	if (more) {
 	    cfg_init(cf_top);
@@ -828,6 +857,7 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 	else if (temp<0) die("Default image doesn't exist.");
 	
 	check_fallback();
+	check_unattended();
 	
 	if (verbose>=2) dump_serial_nos();
 	if (do_md_install) raid_final();
