@@ -1,7 +1,7 @@
-/* lilo.h  -  LILO constants */
-/*
+/* lilo.h  -  LILO constants
+
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -51,6 +51,17 @@ source directory.
 
 #endif
 
+#ifdef HAS_VERSION_H
+#include <linux/version.h>
+#endif
+
+#ifndef LINUX_VERSION_CODE
+#define LINUX_VERSION_CODE 0
+#endif
+#ifndef KERNEL_VERSION
+#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
+#endif
+
 #define nelem(a) (sizeof(a)/sizeof((a)[0]))
 #define S2(x) #x
 #define S(x) S2(x)
@@ -118,6 +129,12 @@ source directory.
 #define MAJOR_IBM_iSER	112 /* IBM iSeries virtual disk */
 #define MAJOR_HPT370	114 /* HPT370 controller */
 #define MAJOR_EVMS	117 /* Enterprise Volume Management System */
+#define MAJOR_SD9	128 /* SCSI disks 129     */
+#define MAJOR_SD16	135 /* SCSI disks    -255 */
+#define MAJOR_DAC960_8	136 /* Ninth Mylex DAC960 PCI RAID controller */
+#define MAJOR_EMD	153 /* Enhanced multi-disk RAID sets */
+#define MAJOR_SATA	160 /* Carmel SATA Disk on first 8-port controller */
+#define MAJOR_SATA2	161 /* Carmel SATA Disk on 2nd 8-port controller */
 
 #define MAX_TOKEN	1023 /* max device Token length */
 #define MAX_IMAGE_NAME	15 /* maximum name length (w/o terminating NUL) */
@@ -137,12 +154,12 @@ source directory.
 #ifdef LCF_DSECS
 #define MAX_DESCR_SECS LCF_DSECS
 #else
-#define MAX_DESCR_SECS 2	/* maximum # of descriptor sectors (8 max) */
+#define MAX_DESCR_SECS 2	/* maximum # of descriptor sectors */
 #endif
 #if MAX_DESCR_SECS > MAX_DESCR_SECTORS
 # error "Maximum DSECS=x exceeded."
 #endif
-#define MAX_IMAGES      ((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(long)-1)/sizeof(IMAGE_DESCR))
+#define MAX_IMAGES      ((int)((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(int)-1)/sizeof(IMAGE_DESCR)))
 			  /* maximum number of images */
 
 #define SECTOR_SIZE	512 /* disk sector size */
@@ -166,12 +183,18 @@ source directory.
 				   hacks) scream if the boot sector gets any
 				   bigger -- (22.5 - we now use those hacks) */
 
-#define BOOT_SIGNATURE	0xaa55	/* boot signature */
+#define BOOT_SIGNATURE	0xAA55	/* boot signature */
 #define BOOT_SIG_OFFSET	510	/* boot signature offset */
 #define MAGIC_SERIAL	0xC9CF	/* LILO installed serial number */
 #define PRIME		271	/* prime number used for serial no generation */
 #define SMALL_PRIME     17	/* another prime, but a small one */
-#define MAX_BIOS_DEVICES  16	/* max hard disk devices */
+#if VERSION_MINOR>90
+#define MAX_BIOS_DEVICES  2	/* max hard disk devices used by BIOS */
+#define MAX_DEVICES	4	/* max hard disk devices, total */
+#else
+#define MAX_BIOS_DEVICES  16	/* max hard disk devices used by BIOS */
+#define MAX_DEVICES	64	/* max hard disk devices, total */
+#endif
 #define MAX_RAID	30	/* max number of RAID disks in a set */
 #if 0
 #define MAX_RAID_DEVICES  6	/* max raid devices reported to second.S */
@@ -237,7 +260,8 @@ source directory.
  *	0x20	 LBA32 address (24-bits) + (8-bit) high nibble (implied)
  *
 */
-#define DEV_MASK        0x8f    /* mask to actual device code */
+#define DEV_MASK_EXP	0x80+MAX_BIOS_DEVICES-1
+#define DEV_MASK	(DEV_MASK_EXP)
 
 #define	EX_OFF		SETUP_STACKSIZE-8+SSDIFF /* external parameter block */
 #define EX_DL_MAG	0xfe	/* magic number in DL */
@@ -270,7 +294,8 @@ source directory.
 #endif
 
 /* these are the second-stage specific flags */
-#define FLAG2_EL_TORITO	2	/* El Torito format bootable CD */
+#define FLAG2_EL_TORITO	 2	/* El Torito format bootable CD */
+#define FLAG2_UNATTENDED 4	/* Unattended booting option */
 
 /* these are the descriptor flags */
 #define FLAG_VGA	1	/* override VGA mode */
@@ -344,6 +369,7 @@ SECTOR_SIZE_asm = SECTOR_SIZE		! **
 MAX_MENU_TITLE_asm = MAX_MENU_TITLE	! **
 MAX_BIOS_DEVICES_asm = MAX_BIOS_DEVICES	! **
 MAX_RAID_DEVICES_asm = MAX_RAID_DEVICES	! **
+DEV_MASK_asm = DEV_MASK_EXP		! **
 
 
 STACKSEG  = 0x9000	     ! MUST == INITSEG for kernel 2.0.36 (and others?)
