@@ -1,7 +1,7 @@
 /* map.c  -  Map file creation */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -10,6 +10,7 @@ source directory.
 */
 
 
+#define _GNU_SOURCE
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -136,7 +137,7 @@ void map_descrs(DESCR_SECTORS *descr, SECTOR_ADDR* addr, SECTOR_ADDR* dflcmd)
 
     if (verbose >= 2) {
 	if (fstat(map_file,&st) < 0) pdie("fstat map file");
-	printf("Map file size: %ld bytes.\n",(long) st.st_size);
+	printf("Map file size: %d bytes.\n",(int) st.st_size);
     }
     if (last!=lseek(map_file, last, SEEK_SET)) pdie("lseek map file to end");
 }
@@ -336,12 +337,12 @@ int map_end_section(SECTOR_ADDR *addr,int dont_compact)
 }
 
 #ifdef LCF_FIRST6
-static long sa6_from_sa(SECTOR_ADDR6 *sa6, SECTOR_ADDR *sa)
+static int sa6_from_sa(SECTOR_ADDR6 *sa6, SECTOR_ADDR *sa)
 {
     static unsigned char hinib = 0;
     int count;
     int flags;
-    long sector;
+    int sector;
     
     sa6->device = sa->device & DEV_MASK;
     flags = sa6->flags = sa->device & ~DEV_MASK;
@@ -349,7 +350,7 @@ static long sa6_from_sa(SECTOR_ADDR6 *sa6, SECTOR_ADDR *sa)
     
     if ((flags & (LBA32_FLAG|LINEAR_FLAG)) == 0) {
 /* pure geometric addressing */
-	sector = *(unsigned long*)&(sa->sector);
+	sector = *(unsigned int*)&(sa->sector);
     }
     else {
 	if (flags & LBA32_FLAG) {
@@ -382,7 +383,7 @@ int map_write(SECTOR_ADDR *list,int max_len,int terminate)
     int sectors;
 #ifdef LCF_FIRST6
     SECTOR_ADDR6 sa6tem, *list6 = (void*)list;
-    unsigned long *list4 = (void*)list;
+    unsigned int *list4 = (void*)list;
 #endif
 
     sectors = 0;
