@@ -1,7 +1,7 @@
 /* partition.c  -  Partition table handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -9,9 +9,8 @@ source directory.
 
 */
 
-
+#define _GNU_SOURCE
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -19,10 +18,9 @@ source directory.
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <asm/unistd.h>
 #include <limits.h>
-/*#include <linux/fs.h>*/
 #include <time.h>
+#include <dirent.h>
 #include "config.h"
 #include "lilo.h"
 #include "common.h"
@@ -33,19 +31,30 @@ source directory.
 #include "boot.h"
 #include "loader.h"
 
-
+#if __GLIBC__ < 2 || __GLIBC_MINOR__ < 1
+#if defined(_syscall5) && defined(__NR__llseek)
        
-       _syscall5(int,  _llseek,  unsigned int,  fd, unsigned long, hi,
-           unsigned long, lo, loff_t *, res, unsigned int, wh);
-       int _llseek(unsigned int fd,  unsigned  long  offset_high,
-           unsigned  long  offset_low,  loff_t * result, unsigned int whence);
+       _syscall5(int,  _llseek,  unsigned int,  fd, unsigned int, hi,
+           unsigned int, lo, lloff_t *, res, unsigned int, wh);
+       int _llseek(unsigned int fd,  unsigned  int  offset_high,
+           unsigned  int  offset_low,  lloff_t * result, unsigned int whence);
 
-       loff_t llseek(unsigned int fd, loff_t offs, unsigned int whence)
-       { loff_t res;
+       lloff_t lseek64(unsigned int fd, lloff_t offs, unsigned int whence)
+       { lloff_t res;
        	   return _llseek(fd, offs>>32, offs, &res, whence) < 0  ?
-       			 (loff_t)(-1) : res;
+       			 (lloff_t)(-1) : res;
        }
 
+#else
+/* last ditch attempt on small disks, and very old systems */
+# warning "*****************************************"
+# warning "***** no 64 bit lseek is available ******"
+# warning "***** using 23 bit sector addresses *****"
+# warning "*****************************************"
+# define lseek64 lseek
+#endif
+#endif
+
 static
 int anywhere(void *buf, unsigned char *str)
 {
@@ -113,7 +122,7 @@ if ( !(do_md_install && extra==X_MBR_ONLY) ) {
     }
     
 /* check for SWAP -- last check, as 'bs' is overwritten */
-    else if (*(long*)bs.sector == 0xFFFFFFFEUL) {
+    else if (*(int*)bs.sector == 0xFFFFFFFEU) {
 	if (lseek(fd, (PAGE_SIZE)-SECTOR_SIZE, SEEK_SET) != (PAGE_SIZE)-SECTOR_SIZE)
 	    pdie("part_nowrite lseek:");
 	if (SECTOR_SIZE != read(fd, bs.sector, sizeof(bs)) ) pdie("part_nowrite swap check:");
@@ -139,7 +148,7 @@ void part_verify(int dev_nr,int type)
     DEVICE dev;
     char backup_file[PATH_MAX+1];
     int fd, bck_file, part, size, lin_3d, cyl;
-    unsigned long second, base;
+    unsigned int second, base;
     struct partition part_table[PART_MAX];
     int mask, i, pe, Linux, dos, mbr;
     unsigned short boot_sig;
@@ -160,7 +169,7 @@ void part_verify(int dev_nr,int type)
     if (type) {
 	if (lseek(fd, 0L, SEEK_SET) != 0 ||
 	    read(fd, &bs, sizeof(bs)) != sizeof(bs) ) pdie("bs read");
-	if (*(long*)bs.signature==EX_MAG_HL) mbr = bs.stage;
+	if (*(int*)bs.signature==EX_MAG_HL) mbr = bs.stage;
 	else mbr = STAGE_MBR;
     } else mbr = STAGE_MBR;
 #endif
@@ -182,8 +191,8 @@ void part_verify(int dev_nr,int type)
     }
     i=5;
     while (i<=pe && base) {
-        if (llseek(fd, LLSECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
-            die("secondary llseek failed");
+        if (lseek64(fd, LLSECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
+            die("secondary lseek64 failed");
 	if (read(fd, part_table, sizeof(part_table)) != sizeof(part_table)) die("secondary read pt failed");
 	if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
 	    boot_sig != BOOT_SIGNATURE ) die("read second boot signature failed");
@@ -302,7 +311,7 @@ void do_cr_reset(void)
 static unsigned char cvt_byte(const char *s)
 {
     char *end;
-    unsigned long value;
+    unsigned int value;
 
     value = strtoul(s,&end,0);
     if (value > 255 || *end) cfg_error("\"%s\" is not a byte value",s);
@@ -607,7 +616,7 @@ void do_activate(char *part, char *which)
     	if (pt[count].sys_ind && pt[count].boot_ind != flag) {
     	    pt[count].boot_ind = flag;
 	    printf("pt[%d] -> %2x\n", count+1, (int)flag);
-	    if (llseek(fd, daddr[count], SEEK_SET) < 0) die("PT llseek failed");
+	    if (lseek64(fd, daddr[count], SEEK_SET) < 0) die("PT lseek64 failed");
 	    if (!test)
 	    if (write(fd, &pt[count], sizeof(pt[0])) != sizeof(pt[0]) )
 		die("PT write failure");
@@ -741,7 +750,7 @@ void do_install_mbr(char *part, char *what)
 
 
 /* partition table read */
-int read_partitions(char *part, int max, long *volid,
+int read_partitions(char *part, int max, int *volid,
 		struct partition *p, long long *where)
 {
     int fd, i;
@@ -791,8 +800,8 @@ int read_partitions(char *part, int max, long *volid,
     if (max>0)
     while (base) {
     	daddr = LLSECTORSIZE*(base+second) + PART_TABLE_OFFSET;
-        if (llseek(fd, daddr, SEEK_SET) < 0)
-            die("secondary llseek failed");
+        if (lseek64(fd, daddr, SEEK_SET) < 0)
+            die("secondary lseek64 failed");
 	if (read(fd, pt, sizeof(pt)) != sizeof(pt)) die("secondary read pt failed");
 	if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
 	    boot_sig != BOOT_SIGNATURE ) die("read second boot signature failed");
