@@ -1,6 +1,6 @@
 /* probe.c -- BIOS probes */
 /*
-Copyright 1999-2003 John Coffman.
+Copyright 1999-2004 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -12,6 +12,7 @@ source directory.
 #define BITMAP 0	/* change to 1 when do_bitmap is filled in */
 #define VOLID  1	/* change to 1 when do_volid is filled in */
 
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -21,7 +22,7 @@ source directory.
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <asm/unistd.h>
+#include <linux/unistd.h>
 #include "lilo.h"
 #include "common.h"
 #include "device.h"
@@ -190,7 +191,7 @@ int fetch(void)
         buf.s5.version > (short)(PROBE_VERSION)) return buf_valid=3;
     got = buf.s5.length;
     if (got > sizeof(buf.b) || got < sizeof(buf.s4)) return buf_valid=4;
-    if (*(long*)buf.s5.checksum != crc32((char*)&buf.s5 + 4, got-4, CRC_POLY1))
+    if (*(int*)buf.s5.checksum != crc32((char*)&buf.s5 + 4, got-4, CRC_POLY1))
     	return buf_valid=5;
 
     if (buf.s5.version == 4) {
@@ -229,7 +230,7 @@ int purge(void)
     if ((i=fetch())) return i;	/* return error from fetch */
     
     i = buf.s5.length;
-    *(long*)buf.s5.checksum = crc32((char*)&buf.s5 + 4, i-4, CRC_POLY1);
+    *(int*)buf.s5.checksum = crc32((char*)&buf.s5 + 4, i-4, CRC_POLY1);
     
     if ((fd=open("/dev/mem", O_WRONLY)) < 0) pdie("purge: can't open /dev/mem");
     if (lseek(fd, seek, SEEK_SET) != seek) pdie("purge: ");
@@ -245,8 +246,10 @@ int purge(void)
 
 static int notice(int needed)
 {
-    if (fetch() || buf.s5.version < needed) {
-	printf(
+    int f = fetch();
+    
+    if (f || buf.s5.version < needed) {
+	printf( f==1 ?  "Only 'root' may do this.\n\n" :
 #if 0
 		"This information request requires that you previously booted your system\n"
 		"using LILO version %s or later.  These versions of the LILO boot\n"
@@ -306,7 +309,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
     floppy_t *fd;
     int i;
     struct partition *pt_base;
-    unsigned long total;
+    unsigned int total;
 
 #if 0
     if((i=fetch())) {
@@ -515,19 +518,19 @@ static void do_ebda(void)
 
 }
 
-static char *number(unsigned long n)
+static char *number(unsigned int n)
 {
-    unsigned long k = 1000000000UL;	/* 10^9 */
+    unsigned int k = 1000000000UL;	/* 10^9 */
     static char s[16];
     char *cp = s;
     
     while (n<k && k>1) k /= 1000UL;
-    sprintf(cp,"%lu",n/k);
+    sprintf(cp,"%u",n/k);
     n %= k;
     k /= 1000UL;
     while (k) {
 	while (*++cp) ;
-	sprintf(cp,",%03lu",n/k);
+	sprintf(cp,",%03u",n/k);
 	n %= k;
 	k /= 1000UL;
     }
@@ -541,7 +544,7 @@ static void print_geom(int dr, struct disk_geom geom)
     char ch_ser[24] = { 0 };
     char *sz = "KMGT";
     char *sp = sz;
-    unsigned long n, m=0;
+    unsigned int n, m=0;
 
 #ifdef DEBUG
     if (!dr) geom.n_total_blocks = 4000000000UL;	/* 2.09Tb */
@@ -557,14 +560,14 @@ static void print_geom(int dr, struct disk_geom geom)
 	n /= 1000UL;
 	sp++;
     }
-    if (m) printf("\t(%3lu.%02lu%cb", n, m, *sp);
-    else printf("\t(%3lu%cb", n, *sp);
+    if (m) printf("\t(%3u.%02u%cb", n, m, *sp);
+    else printf("\t(%3u%cb", n, *sp);
     
     printf("%14s sectors)", number(geom.n_total_blocks));
     if (geom.EDD_flags & EDD_PACKET) {
 /*	printf("\tEDD packet calls allowed");	*/
 	printf("\tLBA32 supported (EDD bios)");
-    } else printf("\tC:H:S supported (IBM PC bios)");
+    } else printf("\tC:H:S supported (PC bios)");
     printf("\n");
 }
 
@@ -647,7 +650,7 @@ static void print_pt(int index, struct partition pt)
 static void do_table(char *part)
 {
     struct partition pt [PART_MAX_MAX+1];
-    long volid;
+    int volid;
     long long where[PART_MAX_MAX+1];
     int i,j;
     int extd = (extended_pt || verbose>0);
@@ -972,8 +975,8 @@ int bios_device(GEOMETRY *geo, int device)
     while (--bios >= 0x80) {
 	get_geom(bios, &bdata);
 	if (verbose>=4) {
-		printf("bios_dev: (0x%02X)  vol-ID=%08X  *PT=%08X\n",
-			bios, bdata.serial_no, (int)bdata.pt);
+		printf("bios_dev: (0x%02X)  vol-ID=%08X  *PT=%08lX\n",
+			bios, bdata.serial_no, (long)bdata.pt);
 #ifdef DEBUG
 		dump_pt((void*)bdata.pt);
 #endif
@@ -1000,7 +1003,7 @@ int bios_device(GEOMETRY *geo, int device)
 	get_geom(bios1, &bdata);
 	if (  (geo->sectors && geo->sectors!=bdata.n_sect)  ||
 	       (geo->heads && geo->heads!=bdata.n_head)  )  {
-	    unsigned long nblocks = geo->cylinders * geo->heads * geo->sectors;
+	    unsigned int nblocks = geo->cylinders * geo->heads * geo->sectors;
 
 	    if (verbose>0 && !nowarn && !(warned[bios1-0x80]&2) ) {
 		fprintf(errstd,"Warning: Kernel & BIOS return differing head/sector geometries for device 0x%02X\n", bios1);
@@ -1119,7 +1122,7 @@ static void do_volid(void)
 {
     int bios, n, i, k, sv, nv, unique;
     struct disk_geom geom;
-    unsigned long serial[MAX_BIOS_DEVICES];
+    unsigned int serial[MAX_BIOS_DEVICES];
     int uniq[MAX_BIOS_DEVICES], valid[MAX_BIOS_DEVICES];
 
     if (notice(4)) exit(0);
@@ -1143,7 +1146,7 @@ static void do_volid(void)
 	unique &= uq;
 	n++;
 
-	printf("  0x%02X     %08lX %s%s\n", i+0x80, serial[i],
+	printf("  0x%02X     %08X %s%s\n", i+0x80, serial[i],
 		uq ? "" : "*", sv ? "" : "-");
     }
 
